@@ -31,14 +31,17 @@
 #define DATABASE_CURRENT_VERSION 1
 #define DEFAULT_INTERVAL 2
 #define APPLICATION_NAME "Wally"
-#define APPLICATION_VERSION "2.3.2"
+#define APPLICATION_VERSION "2.4.2"
 #define RUN_PATH "run"
 #define PID_FILENAME "wally.pid"
 #define MAX_SIZE_FAILURES 20
 #define TIMEOUT 30000
 
-#ifdef Q_WS_WIN
+#if defined(Q_WS_WIN) || defined(Q_WS_PM)
   #include <fstream>
+#endif
+
+#ifdef Q_WS_WIN
   #include <windows.h>
 #endif
 
@@ -48,9 +51,15 @@ namespace Wally
   {
     Q_OBJECT
 
+  public:
+    enum Position { Centered = 0, Tiled = 1, CenterTiled = 2, CenteredMaxpect = 3,
+                    TiledMaxpect = 4, Scaled = 5, CenteredAutoFit = 6, ScaleAndCrop = 7,
+                    SymmetricalTiled = 8, MirroredTiled = 9, SymmetricalMirroredTiled = 10 };
+
+  private:
     #ifdef Q_WS_X11
       enum WindowManager { wmUnknown, wmKDE3, wmKDE4, wmGnome, wmXfce, wmFluxbox,
-                           wmBlackbox, wmWindowMaker, wmFVWM };
+                           wmBlackbox, wmWindowMaker, wmFVWM, wmGnomeShell };
     #endif
 
     enum States { sIdle, sSelectNextEngineFirst, sInitEngine, sSearchRequest,
@@ -65,7 +74,7 @@ namespace Wally
     bool captureKeymaps;
     bool dontSwitchState;
 
-  #ifdef Q_WS_WIN
+  #if defined(Q_WS_WIN) || defined(Q_WS_PM)
     std::ofstream fout;
     bool adWarningFired;
   #endif
@@ -112,6 +121,9 @@ namespace Wally
     QAction *chineseLanguageAction;
     QAction *catalanLanguageAction;
     QAction *greekLanguageAction;
+    QAction *koreanLanguageAction;
+    QAction *hungarianLanguageAction;
+    QAction *danishLanguageAction;
     QWidget *hiddenWidgetForLanguagesActions;
     QList<QAction *> languagesActions;
     QSignalMapper *languagesSignalMapper;
@@ -130,9 +142,13 @@ namespace Wally
     QFileInfo currentFile;
     PhotoInfo currentPhotoInfo;
     bool isSizeValid(const QSize &size) const;
-    bool isCenterInsideDesktop(QPoint center, QSize pixmapSize, QSize desktopSize);
-    void recFillCentersList(QSize desktopSize, QSize pixmapSize, QList<QPoint> &centers, QPoint center);
-    void fillCentersList(QSize desktopSize, QSize pixmapSize, QList<QPoint> &centers);
+    static bool isCenterStored(const QPoint &center, const QList< QPair< QPoint,QPair<bool,bool> > > &centers);
+    bool isCenterInsideDesktop(const QPoint &center, const QSize &pixmapSize, const QSize &desktopSize);
+    void recFillCentersList(const QSize &desktopSize, const QSize &pixmapSize,
+                            QList< QPair< QPoint,QPair<bool,bool> > > &centers, const QPoint &center,
+                            bool horizontalFlip, bool verticalFlip);
+    void fillCentersList(const QSize &desktopSize, QSize pixmapSize,
+                         QList< QPair< QPoint,QPair<bool,bool> > > &centers);
     QFileInfo adaptPhoto(const QFileInfo &file, const QString &format, const PhotoInfo &info);
     void tryCompletePhotoInfo();
 
@@ -142,9 +158,7 @@ namespace Wally
     void applySettings(WallySettings newSettings);
 
     void checkPlayEnableable();
-  #ifndef Q_WS_WIN
     bool isPIDBelongingToWally(int pid) const;
-  #endif
     void checkSingleInstance();
 
     void retranslateMenu();
@@ -195,6 +209,7 @@ namespace Wally
     QString tempStorageDir() const { return _tempStorageDir; }
 
     void initialize();
+    void loadLanguages();
 
     void addEngine(Engine::Core *e);
 
