@@ -17,9 +17,6 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
-#include <QtCore>
-#include <QtGui>
-
 #include "utils.h"
 #include "gui.h"
 #include "about.h"
@@ -57,7 +54,7 @@ AboutDialog::AboutDialog(QWidget *parent) : Gui::Dialog(Gui::Dialog::TopOfScreen
   twAdditionalInfo->setItem(1,1,new QTableWidgetItem(tr("not present")));
 #endif
   twTranslators->setColumnCount(2);
-  twTranslators->setRowCount(12);
+  twTranslators->setRowCount(15);
   twTranslators->setItem(0,0,new QTableWidgetItem(QIcon(":/images/uk"),tr("English")));
   twTranslators->setItem(0,1,new QTableWidgetItem("Antonio Di Monaco"));
   twTranslators->setItem(1,0,new QTableWidgetItem(QIcon(":/images/italy"),tr("Italian")));
@@ -82,6 +79,12 @@ AboutDialog::AboutDialog(QWidget *parent) : Gui::Dialog(Gui::Dialog::TopOfScreen
   twTranslators->setItem(10,1,new QTableWidgetItem(QString::fromUtf8("\115\151\161\165\145\154\40\122\303\240\155\151\141")));
   twTranslators->setItem(11,0,new QTableWidgetItem(QIcon(":/images/greece"),tr("Greek")));
   twTranslators->setItem(11,1,new QTableWidgetItem(QString::fromUtf8("\316\230\317\211\316\274\316\254\317\202\40\316\244\317\203\316\254\316\262\316\261\316\273\316\277\317\202")));
+  twTranslators->setItem(12,0,new QTableWidgetItem(QIcon(":/images/korea"),tr("Korean")));
+  twTranslators->setItem(12,1,new QTableWidgetItem(QString::fromUtf8("\354\235\264\353\215\225\355\235\254")));
+  twTranslators->setItem(13,0,new QTableWidgetItem(QIcon(":/images/hungary"),tr("Hungarian")));
+  twTranslators->setItem(13,1,new QTableWidgetItem(QString::fromUtf8("\122\165\155\303\241\156\40\123\303\241\156\144\157\162")));
+  twTranslators->setItem(14,0,new QTableWidgetItem(QIcon(":/images/denmark"),tr("Danish")));
+  twTranslators->setItem(14,1,new QTableWidgetItem("Michael Borries"));
   twTranslators->sortItems(0);
 
   twSupportedImageFormats->resizeColumnsToContents();
