@@ -22,10 +22,8 @@ TRANSLATOR Wally::Application
  */
 
 #include <iostream>
-#include <QtCore>
 #include <QtAlgorithms>
 #include <QtNetwork>
-#include <QtSql>
 #include <QtDebug>
 
 #include "exif.h"
@@ -40,6 +38,7 @@ TRANSLATOR Wally::Application
   #include <QX11Info>
   #include <X11/Xlib.h>
   #include <X11/keysym.h>
+  #include <QtDBus>
 #endif
 
 #ifdef Q_WS_WIN
@@ -55,6 +54,12 @@ TRANSLATOR Wally::Application
   #include <sys/sysctl.h>
 #endif
 
+#ifdef Q_WS_PM
+  #define INCL_WINWORKPLACE
+  #include <os2.h>
+  #include <os2emx.h>
+#endif
+
 static const char *statesStrs[] = { "sIdle", "sSelectNextEngineFirst", "sInitEngine",
                                     "sSearchRequest", "sSelectNextEngineNext", "sSizeRequest",
                                     "sDownload", "sInfoRequest", "sCancel", "sAbort" };
@@ -75,12 +80,13 @@ Application::Application(int &argc, char **argv) : QApplication(argc,argv)
   languagesMenu = contextMenu = cancelContextMenu = 0;
   dontSwitchState = singleInstance = commandLineHelp = playWasActive = playTimerWasActive = false;
   hiddenWidgetForLanguagesActions = 0;
+  playAction = 0;
 
   qInstallMsgHandler(printDebugMsg);
 
   if (debugMode)
   {
-  #ifdef Q_WS_WIN
+  #if defined(Q_WS_WIN) || defined(Q_WS_PM)
     fout.open("WallyDebug.txt");
     std::cout.rdbuf(fout.rdbuf());
   #endif
@@ -115,6 +121,8 @@ Application::Application(int &argc, char **argv) : QApplication(argc,argv)
       windowManager = wmBlackbox;
     else if (!wmStr.compare("WindowMaker",Qt::CaseInsensitive))
       windowManager = wmWindowMaker;
+    else if (!wmStr.compare("GnomeShell",Qt::CaseInsensitive))
+      windowManager = wmGnomeShell;
   }
   #endif
 
@@ -125,7 +133,7 @@ Application::Application(int &argc, char **argv) : QApplication(argc,argv)
                       "\n\t-forcewm <wm>\t" +
                       "Use specific window manager (Linux only)\n\n\t" +
                       "where <wm> can be:\tKDE3, KDE4, Gnome, XFCE, FluxBox,\n" +
-                      "\t\t\t\tFVWM, BlackBox, WindowMaker";
+                      "\t\t\t\tFVWM, BlackBox, WindowMaker, GnomeShell";
 
     std::cout << APPLICATION_NAME << " " << APPLICATION_VERSION << std::endl << std::endl;
     std::cout << help.constData() << std::endl;
@@ -158,32 +166,18 @@ Application::~Application()
   if (engines.size())
     qDeleteAll(engines);
 
-#ifndef Q_WS_WIN
+#if !defined(Q_WS_WIN) && !defined(Q_WS_PM)
   QFile::remove(_tempStorageDir + "/" + RUN_PATH + "/" + PID_FILENAME);
 #endif
 
-#ifdef Q_WS_WIN
+#if defined(Q_WS_WIN) || defined(Q_WS_PM)
   if (debugMode)
     fout.close();
 #endif
 }
 
-void Application::initialize()
+void Application::loadLanguages()
 {
-  setupTrayIcon();
-  setupActions();
-  setupMenu();
-  setQuitOnLastWindowClosed(false);
-  qsrand(QDateTime::currentDateTime().toTime_t());
-
-#ifdef Q_WS_X11
-  QDir(QDir::homePath()).mkdir(TEMP_PATH);
-  _tempStorageDir = QDir::homePath() + "/" + TEMP_PATH;
-#else
-  QDir::home().root().mkpath(QDesktopServices::storageLocation(QDesktopServices::DataLocation));
-  _tempStorageDir = QDesktopServices::storageLocation(QDesktopServices::DataLocation);
-#endif
-
   QSettings settings;
   QTranslator *translator;
 
@@ -231,10 +225,39 @@ void Application::initialize()
   translator->load(":/languages/greek");
   translators[QLocale(QLocale::Greek,QLocale::Greece).name()] = translator;
 
+  translator = new QTranslator(this);
+  translator->load(":/languages/korean");
+  translators[QLocale(QLocale::Korean,QLocale::RepublicOfKorea).name()] = translator;
+
+  translator = new QTranslator(this);
+  translator->load(":/languages/hungarian");
+  translators[QLocale(QLocale::Hungarian,QLocale::Hungary).name()] = translator;
+
+  translator = new QTranslator(this);
+  translator->load(":/languages/danish");
+  translators[QLocale(QLocale::Danish,QLocale::Denmark).name()] = translator;
+
   currentLocale = QLocale::English;
   settings.beginGroup(MAIN_SECTION);
   changeLanguage(settings.value(LOCALE,QLocale::system().name()).toString());
   settings.endGroup();
+}
+
+void Application::initialize()
+{
+  setupTrayIcon();
+  setupActions();
+  setupMenu();
+  setQuitOnLastWindowClosed(false);
+  qsrand(QDateTime::currentDateTime().toTime_t());
+
+#ifdef Q_WS_X11
+  QDir(QDir::homePath()).mkdir(TEMP_PATH);
+  _tempStorageDir = QDir::homePath() + "/" + TEMP_PATH;
+#else
+  QDir::home().root().mkpath(QDesktopServices::storageLocation(QDesktopServices::DataLocation));
+  _tempStorageDir = QDesktopServices::storageLocation(QDesktopServices::DataLocation);
+#endif
 
   currentState = sIdle;
 
@@ -253,6 +276,8 @@ void Application::initialize()
     #endif
   }
 
+  retranslateMenu();
+
   #ifdef Q_WS_X11
     autoDetectWM();
   #endif
@@ -276,13 +301,14 @@ void Application::printDebugMsg(QtMsgType type, const char *msg)
   }
 }
 
-#ifndef Q_WS_WIN
 bool Application::isPIDBelongingToWally(int pid) const
 {
 #ifdef Q_WS_X11
   return QFileInfo(QString("/proc/") + QString::number(pid) +
                    "/exe").symLinkTarget().contains(APPLICATION_NAME,Qt::CaseInsensitive);
-#else
+#endif
+
+#ifdef Q_WS_MAC
   struct kinfo_proc *result, *ptr;
   int name[] = { CTL_KERN, KERN_PROC, KERN_PROC_ALL, 0 };
   size_t length, i;
@@ -318,16 +344,25 @@ bool Application::isPIDBelongingToWally(int pid) const
     return true;
   }
 #endif
-}
+
+#if defined(Q_WS_WIN) || defined(Q_WS_PM)
+  Q_UNUSED(pid)
+
+  return false;
 #endif
+}
 
 void Application::checkSingleInstance()
 {
-#ifdef Q_WS_WIN
+#if defined(Q_WS_WIN) || defined(Q_WS_PM)
 
-  QSharedMemory *sharedMemory = new QSharedMemory(QString(APPLICATION_NAME) +
-                                                  "SingleApplicationKey",this);
-  singleInstance = sharedMemory->create(1);
+  #ifndef QT_NO_SHAREDMEMORY
+    QSharedMemory *sharedMemory = new QSharedMemory(QString(APPLICATION_NAME) +
+                                                    "SingleApplicationKey",this);
+    singleInstance = sharedMemory->create(1);
+  #else
+    singleInstance = true;
+  #endif
 
 #else
 
@@ -525,11 +560,32 @@ void Application::setupActions()
                                     QLocale(QLocale::Greek,QLocale::Greece).name());
   connect(greekLanguageAction,SIGNAL(triggered()),languagesSignalMapper,SLOT(map()));
 
+  koreanLanguageAction = new QAction(QIcon(":/images/korea"),QString(),this);
+  languagesActions << koreanLanguageAction;
+  languagesSignalMapper->setMapping(koreanLanguageAction,
+                                    QLocale(QLocale::Korean,QLocale::RepublicOfKorea).name());
+  connect(koreanLanguageAction,SIGNAL(triggered()),languagesSignalMapper,SLOT(map()));
+
+  hungarianLanguageAction = new QAction(QIcon(":/images/hungary"),QString(),this);
+  languagesActions << hungarianLanguageAction;
+  languagesSignalMapper->setMapping(hungarianLanguageAction,
+                                    QLocale(QLocale::Hungarian,QLocale::Hungary).name());
+  connect(hungarianLanguageAction,SIGNAL(triggered()),languagesSignalMapper,SLOT(map()));
+
+  danishLanguageAction = new QAction(QIcon(":/images/denmark"),QString(),this);
+  languagesActions << danishLanguageAction;
+  languagesSignalMapper->setMapping(danishLanguageAction,
+                                    QLocale(QLocale::Danish,QLocale::Denmark).name());
+  connect(danishLanguageAction,SIGNAL(triggered()),languagesSignalMapper,SLOT(map()));
+
   hiddenWidgetForLanguagesActions->addActions(languagesActions);
 }
 
 void Application::retranslateMenu()
 {
+  if (!playAction)
+    return;
+
   qDebug() << "Application::retranslateMenu()";
 
   cancelAction->setText(tr("Cancel"));
@@ -560,6 +616,9 @@ void Application::retranslateMenu()
   chineseLanguageAction->setText(tr("Chinese"));
   catalanLanguageAction->setText(tr("Catalan"));
   greekLanguageAction->setText(tr("Greek"));
+  koreanLanguageAction->setText(tr("Korean"));
+  hungarianLanguageAction->setText(tr("Hungarian"));
+  danishLanguageAction->setText(tr("Danish"));
 
   languagesMenu->clear();
   qSort(languagesActions.begin(),languagesActions.end(),actionCaseSensitiveLessThan);
@@ -613,6 +672,9 @@ void Application::setupMenu()
   languagesMenu->addAction(chineseLanguageAction);
   languagesMenu->addAction(catalanLanguageAction);
   languagesMenu->addAction(greekLanguageAction);
+  languagesMenu->addAction(koreanLanguageAction);
+  languagesMenu->addAction(hungarianLanguageAction);
+  languagesMenu->addAction(danishLanguageAction);
 
   contextMenu = new QMenu;
   contextMenu->addAction(playAction);
@@ -669,6 +731,20 @@ void Application::autoDetectWM()
     file.next();
   }
 
+  file.toFront();
+  while (file.hasNext() && (windowManager == wmUnknown))
+  {
+    file.peekNext().fileName().toInt(&ok);
+
+    if (ok && QFileInfo(file.peekNext().absoluteFilePath() + "/exe").symLinkTarget().contains("gnome-shell"))
+    {
+      windowManager = wmGnomeShell;
+      debugMsg += "detected GnomeShell";
+    }
+
+    file.next();
+  }
+
   file.toFront();
   while (file.hasNext() && (windowManager == wmUnknown))
   {
@@ -1366,9 +1442,9 @@ void Application::showPhotoOnScreen()
   QString toolTip;
   QStringList filters;
   QList<QFileInfo> files;
+  QFileInfo newFile;
   QFile textFile;
   QTextStream textStream;
-  QFileInfo newFile;
   bool lastWasTitle = false;
 
   qDebug() << "Application::showPhotoOnScreen()";
@@ -1379,14 +1455,16 @@ void Application::showPhotoOnScreen()
     switch (windowManager)
     {
       case wmKDE4:
-        newFile = adaptPhoto(currentFile.absoluteFilePath(),"PNG",currentPhotoInfo);
+        {
+          newFile = adaptPhoto(currentFile.absoluteFilePath(),"PNG",currentPhotoInfo);
 
-        textFile.setFileName(_tempStorageDir + "/wallykde4.img");
-        textFile.open(QIODevice::WriteOnly);
-        textStream.setDevice(&textFile);
-
-        textStream << newFile.absoluteFilePath() << endl;
-        textFile.close();
+          QDBusMessage message = QDBusMessage::createMethodCall("com.BeCrux.WallyPlugin",
+                                                                "/WallyPlugin",
+                                                                QString(),
+                                                                "setWallpaper");
+          message.setArguments(QList< QVariant >() << newFile.absoluteFilePath());
+          QDBusConnection::sessionBus().send(message);
+        }
         break;
 
       case wmKDE3:
@@ -1411,6 +1489,18 @@ void Application::showPhotoOnScreen()
         QProcess::execute("gconftool-2",args);
         break;
 
+      case wmGnomeShell:
+        newFile = adaptPhoto(currentFile.absoluteFilePath(),"PNG",currentPhotoInfo);
+        args << "set" << "org.gnome.desktop.background" << "draw-background" << "true";
+        QProcess::execute("gsettings",args);
+        args.clear();
+        args << "set" << "org.gnome.desktop.background" << "picture-options" << "centered";
+        QProcess::execute("gsettings",args);
+        args.clear();
+        args << "set" << "org.gnome.desktop.background" << "picture-uri" << QUrl::fromLocalFile(newFile.absoluteFilePath()).toString();
+        QProcess::execute("gsettings",args);
+        break;
+
       case wmXfce:
         newFile = adaptPhoto(currentFile.absoluteFilePath(),"PNG",currentPhotoInfo);
         textFile.setFileName(QDir::homePath() + "/.config/xfce4/desktop/backdrops.list");
@@ -1491,15 +1581,36 @@ void Application::showPhotoOnScreen()
     #endif
   #endif
 
+  #ifdef Q_WS_PM
+    newFile = adaptPhoto(currentFile.absoluteFilePath(),"BMP",currentPhotoInfo);
+
+    HOBJECT hobj;
+    const char *objName = "<WP_DESKTOP>";
+
+      if (hobj = WinQueryObject(reinterpret_cast< const unsigned char * > (objName)))
+        WinSetObjectData(hobj,reinterpret_cast< const unsigned char * > ((QString("BACKGROUND=%1,S,1,I;").arg(QDir::toNativeSeparators(newFile.absoluteFilePath()))).toAscii().constData()));
+
+  #endif
+
   #ifdef Q_WS_MAC
     newFile = adaptPhoto(currentFile.absoluteFilePath(),"PNG",currentPhotoInfo);
     QFile scriptFile(_tempStorageDir + "/wally.scpt");
     QTextStream scriptText(&scriptFile);
 
     scriptFile.open(QIODevice::WriteOnly);
-    scriptText << "tell application \"Finder\"" << endl;
-    scriptText << "set desktop picture to POSIX file \"" << newFile.absoluteFilePath() << "\"" << endl;
-    scriptText << "end tell" << endl;
+
+    if (QSysInfo::MacintoshVersion == QSysInfo::MV_10_6)
+    {
+      scriptText << "tell application \"System Events\"" << endl;
+	    scriptText << "set picture of every desktop to POSIX file \"" << newFile.absoluteFilePath() << "\"" << endl;
+      scriptText << "end tell" << endl;
+    }
+    else
+    {
+      scriptText << "tell application \"Finder\"" << endl;
+      scriptText << "set desktop picture to POSIX file \"" << newFile.absoluteFilePath() << "\"" << endl;
+      scriptText << "end tell" << endl;
+    }
     scriptFile.close();
 
     args << QDir::toNativeSeparators(_tempStorageDir + "/wally.scpt");
@@ -1552,6 +1663,9 @@ void Application::showPhotoOnScreen()
 
       trayIcon->setToolTip(toolTip);
     }
+
+    if (settings.value(MAIN_SECTION).value(QUIT_AFTER_BACKGROUND_CHANGE).toBool())
+      QTimer::singleShot(10000,quitAction,SLOT(trigger()));
   }
   catch (...)
   {
@@ -1576,7 +1690,7 @@ void Application::showPhotoOnScreen(const QString &engineName)
   }
 }
 
-bool Application::isCenterInsideDesktop(QPoint center, QSize pixmapSize, QSize desktopSize)
+bool Application::isCenterInsideDesktop(const QPoint &center, const QSize &pixmapSize, const QSize &desktopSize)
 {
   return (((center.x() + pixmapSize.width() / 2) >= 0) &&
           ((center.y() + pixmapSize.height() / 2) >= 0) &&
@@ -1584,84 +1698,72 @@ bool Application::isCenterInsideDesktop(QPoint center, QSize pixmapSize, QSize d
           ((center.y() - pixmapSize.width() / 2) <= desktopSize.height()));
 }
 
-void Application::recFillCentersList(QSize desktopSize, QSize pixmapSize,
-                               QList<QPoint> &centers, QPoint center)
+bool Application::isCenterStored(const QPoint &center, const QList< QPair< QPoint,QPair<bool,bool> > > &centers)
+{
+  bool found = false;
+  QListIterator< QPair< QPoint,QPair<bool,bool> > > c(centers);
+
+  while (c.hasNext() && !(found = (c.next().first == center))) ;
+
+  return found;
+}
+
+void Application::recFillCentersList(const QSize &desktopSize, const QSize &pixmapSize,
+                                     QList< QPair< QPoint,QPair<bool,bool> > > &centers, const QPoint &center,
+                                     bool horizontalFlip, bool verticalFlip)
 {
   if (isCenterInsideDesktop(center,pixmapSize,desktopSize) &&
-      !centers.contains(center))
+      !isCenterStored(center,centers))
   {
-    centers.push_back(center);
+    centers.push_back(qMakePair(center,qMakePair(horizontalFlip,verticalFlip)));
 
-    switch (settings.value(MAIN_SECTION).value(WALLPAPER_POSITION).toInt())
-    {
-      case 1: /* tiled */
-      case 4: /* tiled maxpect */
-        recFillCentersList(desktopSize,pixmapSize,centers,
-                          QPoint(center.x() + pixmapSize.width(),center.y()));
-        recFillCentersList(desktopSize,pixmapSize,centers,
-                          QPoint(center.x(),center.y() + pixmapSize.height()));
-        break;
-
-      case 2: /* center tiled */
-        recFillCentersList(desktopSize,pixmapSize,centers,
-                          QPoint(center.x(),center.y() - pixmapSize.height()));
-        recFillCentersList(desktopSize,pixmapSize,centers,
-                          QPoint(center.x(),center.y() + pixmapSize.height()));
-        recFillCentersList(desktopSize,pixmapSize,centers,
-                          QPoint(center.x() - pixmapSize.width(),center.y()));
-        recFillCentersList(desktopSize,pixmapSize,centers,
-                          QPoint(center.x() + pixmapSize.width(),center.y()));
-        break;
-
-      default:
-        break;
-    }
+    recFillCentersList(desktopSize,pixmapSize,centers,
+                       QPoint(center.x(),center.y() - pixmapSize.height()),horizontalFlip,!verticalFlip);
+    recFillCentersList(desktopSize,pixmapSize,centers,
+                       QPoint(center.x(),center.y() + pixmapSize.height()),horizontalFlip,!verticalFlip);
+    recFillCentersList(desktopSize,pixmapSize,centers,
+                       QPoint(center.x() - pixmapSize.width(),center.y()),!horizontalFlip,verticalFlip);
+    recFillCentersList(desktopSize,pixmapSize,centers,
+                       QPoint(center.x() + pixmapSize.width(),center.y()),!horizontalFlip,verticalFlip);
   }
 }
 
-void Application::fillCentersList(QSize desktopSize, QSize pixmapSize, QList<QPoint> &centers)
+void Application::fillCentersList(const QSize &desktopSize, QSize pixmapSize,
+                                  QList< QPair< QPoint,QPair<bool,bool> > > &centers)
 {
-  double aspectRatio;
-
-  switch (settings.value(MAIN_SECTION).value(WALLPAPER_POSITION).toInt())
+  switch (static_cast<Position> (settings.value(MAIN_SECTION).value(WALLPAPER_POSITION).toInt()))
   {
-    case 0: /* centered */
-    case 3: /* centered maxpect */
-    case 5: /* scaled */
-    case 6: /* centered autofit */
-    case 7: /* scale & crop */
-      centers.push_back(QPoint(desktopSize.width() / 2,desktopSize.height() / 2));
+    case Centered:
+    case CenteredMaxpect:
+    case Scaled:
+    case CenteredAutoFit:
+    case ScaleAndCrop:
+      centers.push_back(qMakePair(QPoint(desktopSize.width() / 2,desktopSize.height() / 2),
+                                  qMakePair(false,false)));
       break;
 
-    case 4: /* tiled maxpect */
-      aspectRatio = static_cast<double> (pixmapSize.width()) /
-                    static_cast<double> (pixmapSize.height());
-      if (aspectRatio >= 1.0)
-        pixmapSize = QSize(desktopSize.width(),
-                           static_cast<int> (static_cast<double> (desktopSize.width()) /
-                                                                  aspectRatio));
-      else
-        pixmapSize = QSize(static_cast<int> (static_cast<double> (desktopSize.height()) *
-                                                                  aspectRatio),
-                           desktopSize.height());
-    case 1: /* tiled or tiled maxpect */
-      centers.push_back(QPoint(pixmapSize.width() / 2,pixmapSize.height() / 2));
+    case TiledMaxpect:
+      pixmapSize.scale(desktopSize,Qt::KeepAspectRatio);
+
+    case Tiled: /* or tiled maxpect */
+    case MirroredTiled:
       recFillCentersList(desktopSize,pixmapSize,centers,
-                         QPoint(pixmapSize.width() / 2 + pixmapSize.width(),pixmapSize.height() / 2));
-      recFillCentersList(desktopSize,pixmapSize,centers,
-                         QPoint(pixmapSize.width() / 2,pixmapSize.height() / 2 + pixmapSize.height()));
+                         QPoint(pixmapSize.width() / 2,pixmapSize.height() / 2),
+                         false,false);
       break;
 
-    case 2: /* center tiled */
-      centers.push_back(QPoint(desktopSize.width() / 2,desktopSize.height() / 2));
+    case CenterTiled:
       recFillCentersList(desktopSize,pixmapSize,centers,
-                         QPoint(desktopSize.width() / 2,desktopSize.height() / 2 - pixmapSize.height()));
+                         QPoint(desktopSize.width() / 2,desktopSize.height() / 2),
+                         false,false);
+      break;
+
+    case SymmetricalTiled:
+    case SymmetricalMirroredTiled:
       recFillCentersList(desktopSize,pixmapSize,centers,
-                         QPoint(desktopSize.width() / 2,desktopSize.height() / 2 + pixmapSize.height()));
-      recFillCentersList(desktopSize,pixmapSize,centers,
-                         QPoint(desktopSize.width() / 2 - pixmapSize.width(),desktopSize.height() / 2));
-      recFillCentersList(desktopSize,pixmapSize,centers,
-                         QPoint(desktopSize.width() / 2 + pixmapSize.width(),desktopSize.height() / 2));
+                         QPoint(desktopSize.width() / 2,desktopSize.height() / 2) -
+                         QPoint(pixmapSize.width() / 2,pixmapSize.height() / 2),
+                         false,false);
       break;
 
     default:
@@ -1720,12 +1822,13 @@ QFileInfo Application::adaptPhoto(const QFileInfo &file, const QString &format,
 {
   int prevHeight;
   QString newFileName;
-  QList<QPoint> centers;
+  QList< QPair< QPoint,QPair<bool,bool> > > centers;
   QRect userRect = (settings.value(MAIN_SECTION).value(USE_FULL_DESKTOP_AREA).toBool())?
                    QApplication::desktop()->screenGeometry() :
                    QApplication::desktop()->availableGeometry();
   QGraphicsScene desktopScene(QApplication::desktop()->screenGeometry());
   QImage image;
+  Position position = static_cast<Position> (settings.value(MAIN_SECTION).value(WALLPAPER_POSITION).toInt());
 
   qDebug() << "Application::adaptPhoto(" << file.absoluteFilePath().toAscii().constData() << ","
            << format.toAscii().constData() << ","
@@ -1758,30 +1861,30 @@ QFileInfo Application::adaptPhoto(const QFileInfo &file, const QString &format,
   painter.setRenderHint(QPainter::Antialiasing,true);
   painter.setRenderHint(QPainter::TextAntialiasing,true);
 
-  switch (settings.value(MAIN_SECTION).value(WALLPAPER_POSITION).toInt())
+  switch (position)
   {
-    case 0: /* centered */
-    case 1: /* tiled */
-    case 2: /* center tiled */
+    case Centered:
+    case Tiled:
+    case CenterTiled:
     default:
       break;
 
-    case 3: /* centered maxpect */
-    case 4: /* tiled maxpect */
+    case CenteredMaxpect:
+    case TiledMaxpect:
       image = image.scaled(userRect.size(),Qt::KeepAspectRatio,Qt::SmoothTransformation);
       break;
 
-    case 5: /* scaled */
+    case Scaled:
       image = image.scaled(userRect.size(),Qt::IgnoreAspectRatio,Qt::SmoothTransformation);
       break;
 
-    case 6: /* centered autofit */
-      if ((image.size().width() * image.size().height()) >
-          (userRect.width() * userRect.height()))
+    case CenteredAutoFit:
+      if ((image.size().width() > userRect.width()) ||
+          (image.size().height() > userRect.height()))
         image = image.scaled(userRect.size(),Qt::KeepAspectRatio,Qt::SmoothTransformation);
       break;
 
-    case 7: /* scale & crop */
+    case ScaleAndCrop:
       image = image.scaled(userRect.size(),Qt::KeepAspectRatioByExpanding,Qt::SmoothTransformation);
       break;
   }
@@ -1808,12 +1911,16 @@ QFileInfo Application::adaptPhoto(const QFileInfo &file, const QString &format,
     desktopScene.setBackgroundBrush(QColor(settings.value(MAIN_SECTION).value(BORDER_COLOR).toString()));
   fillCentersList(userRect.size(),image.size(),centers);
 
-  QListIterator<QPoint> center(centers);
+  QListIterator< QPair< QPoint,QPair<bool,bool> > > center(centers);
   while (center.hasNext())
   {
-    desktopScene.addPixmap(QPixmap::fromImage(image))->setPos(center.peekNext().x() - image.width() / 2 + userRect.left(),
-                                                              center.peekNext().y() - image.height() / 2 + userRect.top());
-    center.next();
+    QPair< QPoint,QPair<bool,bool> > c = center.next();
+    QPoint p(c.first.x() - image.width() / 2 + userRect.left(),
+             c.first.y() - image.height() / 2 + userRect.top());
+    desktopScene.addPixmap(QPixmap::fromImage(image.mirrored(((position == MirroredTiled) ||
+                                                              (position == SymmetricalMirroredTiled)) && c.second.first,
+                                                             ((position == MirroredTiled) ||
+                                                              (position == SymmetricalMirroredTiled)) && c.second.second)))->setPos(p);
   }
 
   if (settings.value(MAIN_SECTION).value(VIEW_INFO_ON_PHOTO).toBool())
@@ -2069,11 +2176,13 @@ void Application::loadSettings()
   qDebug() << "Application::loadSettings()";
 
   sourceSettings.beginGroup(MAIN_SECTION);
+  settings[MAIN_SECTION][DISABLE_SPLASH_SCREEN] = sourceSettings.value(DISABLE_SPLASH_SCREEN,false);
+  settings[MAIN_SECTION][QUIT_AFTER_BACKGROUND_CHANGE] = sourceSettings.value(QUIT_AFTER_BACKGROUND_CHANGE,false);
   settings[MAIN_SECTION][INTERVAL] = sourceSettings.value(INTERVAL,DEFAULT_INTERVAL);
   settings[MAIN_SECTION][INTERVAL_UNIT] = sourceSettings.value(INTERVAL_UNIT,MINUTES_UNIT);
   settings[MAIN_SECTION][BORDER_COLOR] = sourceSettings.value(BORDER_COLOR,QColor(Qt::black).name());
   settings[MAIN_SECTION][AUTO_COLOR] = sourceSettings.value(AUTO_COLOR,false);
-  settings[MAIN_SECTION][WALLPAPER_POSITION] = sourceSettings.value(WALLPAPER_POSITION,0);
+  settings[MAIN_SECTION][WALLPAPER_POSITION] = sourceSettings.value(WALLPAPER_POSITION,static_cast<int> (Centered));
   settings[MAIN_SECTION][SWITCH_ON_PLAY] = sourceSettings.value(SWITCH_ON_PLAY,false);
   settings[MAIN_SECTION][PLAY_ON_START] = sourceSettings.value(PLAY_ON_START,false);
   settings[MAIN_SECTION][RUN_ON_SYS_START] = sourceSettings.value(RUN_ON_SYS_START,false);
@@ -2129,6 +2238,8 @@ void Application::saveSettings()
   qDebug() << "Application::saveSettings()";
 
   destSettings.beginGroup(MAIN_SECTION);
+  destSettings.setValue(DISABLE_SPLASH_SCREEN,settings.value(MAIN_SECTION).value(DISABLE_SPLASH_SCREEN));
+  destSettings.setValue(QUIT_AFTER_BACKGROUND_CHANGE,settings.value(MAIN_SECTION).value(QUIT_AFTER_BACKGROUND_CHANGE));
   destSettings.setValue(INTERVAL,settings.value(MAIN_SECTION).value(INTERVAL));
   destSettings.setValue(INTERVAL_UNIT,settings.value(MAIN_SECTION).value(INTERVAL_UNIT));
   destSettings.setValue(BORDER_COLOR,settings.value(MAIN_SECTION).value(BORDER_COLOR));
